@@ -1,3 +1,6 @@
+#undef G_DISABLE_ASSERT
+#undef G_LOG_DOMAIN
+
 #include <glib.h>
 
 #define RUNS 100
@@ -5,6 +8,7 @@
 G_LOCK_DEFINE_STATIC (thread_counter);
 gulong abs_thread_counter;
 gulong running_thread_counter;
+gulong leftover_task_counter;
 
 void
 thread_pool_func (gpointer a, gpointer b)
@@ -18,6 +22,7 @@ thread_pool_func (gpointer a, gpointer b)
 
   G_LOCK (thread_counter);
   running_thread_counter--;
+  leftover_task_counter--;
   G_UNLOCK (thread_counter);
 }
 
@@ -32,25 +37,23 @@ main (int   argc,
   guint i;
   g_thread_init (NULL);
   
-  pool1 = g_thread_pool_new (thread_pool_func, 3, 0, FALSE, 
-                            G_THREAD_PRIORITY_NORMAL, FALSE, NULL, NULL);
-  pool2 = g_thread_pool_new (thread_pool_func, 5, 0, FALSE, 
-			     G_THREAD_PRIORITY_LOW, FALSE, NULL, NULL);
-  pool3 = g_thread_pool_new (thread_pool_func, 7, 0, FALSE, 
-                             G_THREAD_PRIORITY_LOW, TRUE, NULL, NULL);
+  pool1 = g_thread_pool_new (thread_pool_func, NULL, 3, FALSE, NULL);
+  pool2 = g_thread_pool_new (thread_pool_func, NULL, 5, TRUE, NULL);
+  pool3 = g_thread_pool_new (thread_pool_func, NULL, 7, TRUE, NULL);
 
   for (i = 0; i < RUNS; i++)
     {
       g_thread_pool_push (pool1, GUINT_TO_POINTER (1), NULL);
       g_thread_pool_push (pool2, GUINT_TO_POINTER (1), NULL);
       g_thread_pool_push (pool3, GUINT_TO_POINTER (1), NULL);
+      leftover_task_counter += 3;
     } 
   
-  g_thread_pool_free (pool1, FALSE, TRUE);
+  g_thread_pool_free (pool1, TRUE, TRUE);
   g_thread_pool_free (pool2, FALSE, TRUE);
   g_thread_pool_free (pool3, FALSE, TRUE);
 
-  g_assert (RUNS * 3 == abs_thread_counter);
+  g_assert (RUNS * 3 == abs_thread_counter + leftover_task_counter);
   g_assert (running_thread_counter == 0);  
 #endif
   return 0;
